@@ -1,8 +1,8 @@
 /**
- * Copyright (C) 2002,2003 Alastair McKinstry, <mckinstry@debian.org>
+ * Copyright (C) 2002,2003, 2005 Alastair McKinstry, <mckinstry@debian.org>
  * Released under the GPL
  *
- * $Id: kbd-chooser.c 25774 2005-02-18 13:12:48Z cjwatson $
+ * $Id: kbd-chooser.c 41384 2006-10-07 20:08:50Z bubulle $
  */
 
 #include "config.h"
@@ -26,15 +26,6 @@
 #include "kbd-chooser.h"
 
 
-// TODO Move this into debian-installer.h
-
-#ifndef di_info
-#define di_info(format...)   di_log(DI_LOG_LEVEL_INFO, format)
-#endif
-#ifndef di_debug
-#define di_debug(format...)  di_log(DI_LOG_LEVEL_INFO, format)
-#endif
-
 extern int loadkeys_wrapper (char *map);	// in loadkeys.y
 
 typedef enum { 
@@ -45,7 +36,7 @@ typedef enum {
 
 
 struct debconfclient *
-mydebconf_get (void) {
+mydebconf_client (void) {
 	static struct debconfclient *client = NULL;
 	if (client == NULL)
 		client = debconfclient_new ();
@@ -56,7 +47,7 @@ int
 mydebconf_ask (char *priority, char *template, char **result)
 {
 	int res;
-	struct debconfclient *client = mydebconf_get ();
+	struct debconfclient *client = mydebconf_client ();
 
 	debconf_input (client, priority, template);
 	res = debconf_go (client);
@@ -74,12 +65,12 @@ int
 mydebconf_default_set (char *template, char *value)
 {
 	int res = 0;
-	struct debconfclient *client = mydebconf_get ();
+	struct debconfclient *client = mydebconf_client ();
 	if ((res = debconf_get (client, template)))
 		return res;
 
 	if (client->value == NULL || (strlen (client->value) == 0))
-		res = debconf_set (client, template, strdup (value));
+		res = debconf_set (client, template, value);
 	return res;
 }
 
@@ -97,24 +88,28 @@ int check_dir (const char *dirname)
 
 /**
  * @brief  Do a grep for a string
- * @return 0 if present, 1 if not, -errno if error
+ * @return 0 if not found, -errno if error, 
+ *   and LINESIZE * (line - 1) + pos, when found at line, pos.
  */
 int
 grep (const char *file, const char *string)
 {
 	FILE *fp = fopen (file, "r");
 	char buf[LINESIZE];
+	char * ret;
+	int lines = 0;
 	if (!fp)
 		return -errno;
 	while (!feof (fp))	{
 		fgets (buf, LINESIZE, fp);
-		if (strstr (buf, string) != NULL)	{
+		if ((ret = strstr (buf, string)) != NULL)	{
 			fclose (fp);
-			return 0;
+			return ((int) (ret - buf) + 1 + (lines * LINESIZE));
 		}
+		lines ++;
 	}
 	fclose (fp);
-	return 1;
+	return 0;
 }
 
 /*
@@ -128,12 +123,14 @@ grep (const char *file, const char *string)
 char *
 locale_get (void)
 {
-	struct debconfclient *client = mydebconf_get ();
+	int ret = 0;
+
+	struct debconfclient *client = mydebconf_client ();
 	// languagechooser sets locale of the form xx_YY
 	// NO encoding used.
 
-	debconf_get (client, "debian-installer/locale");
-	if (client->value && (strlen (client->value) > 0))
+	ret = debconf_get (client, "debian-installer/locale");
+	if ((ret == 0) && client->value && (strlen (client->value) > 0))
 		return strdup(client->value);
 	else
 		return strdup("en_US");
@@ -222,6 +219,7 @@ char *
 insert_description (char *buf, char *description, int *first_entry)
 {
 	char *s = buf;
+	char *t = description;
 
 	if (*first_entry) {
 		*first_entry = 0;
@@ -229,8 +227,11 @@ insert_description (char *buf, char *description, int *first_entry)
 		strcpy (s, ", ");
 		s += 2;
 	}
-	strcpy (s, description);
-	s += strlen (description);
+	while (*t) {
+		if (*t == '\\' || *t == ',')
+			*s++ = '\\';
+		*s++ = *t++;
+	}
 	*s = '\0';
 	return s;
 }
@@ -250,7 +251,7 @@ maplist_select (maplist_t * maplist)
 	// Pick the default
 	for (mp = maplist->maps ; mp != NULL ; mp  = mp->next)	{
 		score = locale_list_compare (mp->langs);
-		if (score > best)      	{
+		if (score > best) {
 			best = score;
 			preferred = mp;
 		}
@@ -294,13 +295,15 @@ maplist_t *maplist_get (const char *name)
  * @brief	Get a keymap in a maplist; create if necessary
  * @list	maplist to search
  * @name	name of list
+ * @langs	if non-NULL, match these languages
  */
-keymap_t *keymap_get (maplist_t * list, char *name)
+keymap_t *keymap_get (maplist_t * list, const char *name, const char *langs)
 {
 	keymap_t *mp;
 
 	for (mp = list->maps ; mp != NULL; mp = mp->next)    {
-		if (strcmp (mp->name, name) == 0)
+		if (strcmp (mp->name, name) == 0 &&
+		    (!langs || strcmp (mp->langs, langs) == 0))
 			break;
 	}
 	if (mp)
@@ -357,7 +360,7 @@ maplist_parse_file (const char *name)
 		*tab2 = '\0';
 		*nl = '\0';
 
-		map = keymap_get (maplist, tab1 + 1);
+		map = keymap_get (maplist, tab1 + 1, buf);
 		if (!map->langs) {	// new keymap
 			map->langs = strdup (buf);
 			map->description = strdup (tab2 + 1);
@@ -447,60 +450,30 @@ keyboards_sort (kbd_t ** keyboards)
 }
 
 /**
- * @brief Get translated contents of a given template.
- * based on code from main-menu. 
- * TODO: merge into the cdebconf library someday
+ * @brief Get template contents (in the current language).
+ * @param template Template name
+ * @param type Kind of entry
+ * @return the translation, to be freed by caller
  */
 char *
-translated_template_get(char *template)
+template_get(const char *template, const char *type)
 {
-	int ret = 0;
-	static char *languages = NULL;
-	char *colon, *lang;
-	struct debconfclient *client = mydebconf_get();
-	
-	if (!languages) {
-		 ret = debconf_get(client,"debian-installer/language");
-		 if (client->value && (ret != 10))
-			 languages = strdup(client->value);
-	}
-	lang = strdup (languages);
-	while  (lang) {
-		char field[128];
-		
-		colon = strchr (lang, ':');
-		if (colon)
-			*colon = '\0';
-		snprintf(field, sizeof (field), "Description-%s.UTF-8", lang);
-		if (!debconf_metaget(client, template, field))  {
-			free (lang);
-			return (strdup(client->value));
-		}
-		lang = (colon) ? colon + 1 : NULL;
-	}
-	// Description must exist.
-	debconf_metaget(client, template, "Description");
-	free(lang);
-        return strdup(client->value);
+	struct debconfclient *client = mydebconf_client();
+
+	/* cdebconf auto-translates */
+	debconf_metaget(client, template, type);
+	return strdup(client->value);
 }
 
 /**
- * @brief discover what subarchitecture we have
- * @return subarch string, or "", freed by caller.
+ * @brief Get translated description for a given template.
+ * @param template Template name
+ * @return the translation, to be freed by caller
  */
-const char *subarch_get (void)
+char *
+description_get(const char *template)
 {
-	static char *subarch = NULL;
-
-	if (!subarch) {
-		struct debconfclient *client = mydebconf_get();
-
-		if (debconf_get (client, "debian-installer/kernel/subarchitecture"))
-			subarch = strdup("");
-		else
-			subarch = strdup(client->value);
-	}
-	return subarch;
+	return template_get(template, "Description");
 }
 
 /**
@@ -512,7 +485,7 @@ keyboards_get (void)
 {
 	static kbd_t *keyboards = NULL, *p = NULL;
 	char buf[25];
-	const char *subarch = subarch_get();
+	const char *subarch = di_system_subarch_analyze();
 
 	if (keyboards != NULL)
 		return keyboards;
@@ -550,39 +523,25 @@ keyboards_get (void)
 		di_error (": No keyboards found\n");
 		exit (6);
 	}
-	// translate the keyboard names
+	// Get the (translated) keyboard names
 	for (p = keyboards; p != NULL; p = p->next) {
 		sprintf(buf, "kbd-chooser/kbd/%s", p->name);
-		p->description = translated_template_get(buf);
+		p->description = description_get(buf);
 	}
 	return keyboards;
 }
 
-/**
- * @brief translate localised keyboard name back to kbd. arch name
- */
-char *keyboard_parse (char *reply)
-{
-	kbd_t *kb;
-	for (kb = keyboards_get(); kb != NULL; kb = kb->next) {
-		if (!strcmp (reply, kb->description))
-			break;
-	}
-	return (kb) ? kb->name : "none";
-}
-
 /**
  * @brief set debian-installer/uml-console as to whether we are using a user mode linux console
- * This is then passed via prebaseconfig to base-config
  * @return 1 if present, 0 if absent, 2 if unknown.
  */
 sercon_state
 check_if_uml_console (void)
 {
 	sercon_state present = SERIAL_UNKNOWN;
-	struct debconfclient *client = mydebconf_get ();
+	struct debconfclient *client = mydebconf_client ();
 
-	if (grep("/proc/cpuinfo", "User Mode Linux") == 0)
+	if (grep("/proc/cpuinfo", "User Mode Linux") > 0)
 		present = SERIAL_PRESENT;
 	else
 		present = SERIAL_ABSENT;
@@ -594,21 +553,25 @@ check_if_uml_console (void)
 
 /**
  * @brief set debian-installer/serial console as to whether we are using a serial console
- * This is then passed via prebaseconfig to base-config
  * @return 1 if present, 0 if absent, 2 if unknown.
  */
 sercon_state
 check_if_serial_console (void)
 {
 	sercon_state present = SERIAL_UNKNOWN;
-	struct debconfclient *client = mydebconf_get ();
+	struct debconfclient *client = mydebconf_client ();
 	int fd;
 	struct serial_struct sr;
+	int rets, ret1, ret2, ret;
 
 	// Some UARTs don't support the TIOCGSERIAL ioctl(), so also
         // try to detect serial console via cmdline
-	if ((grep("/proc/cmdline","console=ttyS") == 0) ||
-	    (grep("/proc/cmdline","console=ttys") == 0)) {
+	ret1 = grep("/proc/cmdline","console=ttyS");
+	ret2 = grep("/proc/cmdline","console=ttys");
+	rets = ret1 > ret2 ? ret1 : ret2;
+	ret = grep("/proc/cmdline","console=tty0");
+
+	if ((rets > 0) && (rets > ret)) {
 		present = SERIAL_PRESENT;
 	} else {
                 fd = open ("/dev/console", O_NONBLOCK);
@@ -662,15 +625,15 @@ int kbdtype_present (kbdtype_t *archlist, char *name) {
  * @return const char *  - priority of question
  */
 char *
-keyboard_select (void)
+keyboard_select (char *curr_arch)
 {
 	kbd_t *kp = NULL, *preferred = NULL;
 	kbdtype_t *archlist = NULL;
-	char buf[LINESIZE], *s = NULL, *none = NULL;
-	int choices = 0, first_entry = 1;
+	char buf_s[LINESIZE], *s = NULL, buf_t[LINESIZE], *t = NULL, *arch_descr = NULL;
+	int choices = 0, first_entry_s = 1, first_entry_t = 1;
 	sercon_state sercon;
 	sercon_state umlcon;
-	struct debconfclient *client = mydebconf_get ();
+	struct debconfclient *client = mydebconf_client ();
 
 	/* k is returned by a method if it is preferred keyboard.
 	 * For 2.4 kernels, we just select one keyboard. 
@@ -678,7 +641,7 @@ keyboard_select (void)
 	 * of keyboards present.
 	 */
 
-	s = buf;
+	s = buf_s; t = buf_t;
 	// Add the keyboards to debconf
 	for (kp = keyboards_get (); kp != NULL; kp = kp->next) {
 		di_info ("keyboard type %s: present: %s \n", kp->name,
@@ -687,7 +650,8 @@ keyboard_select (void)
 		if ((kp->present != FALSE) &&
 		    (kbdtype_present (archlist, kp->name) == FALSE)) {
 			choices++;
-			s = insert_description (s,  kp->description, &first_entry);
+			s = insert_description (s, kp->name, &first_entry_s);
+			t = insert_description (t, kp->description, &first_entry_t);
 			archlist = add_kbdtype (archlist, kp->name);
 			if (strcmp (PREFERRED_KBD, kp->name) == 0) {
 				if ((preferred == NULL) || (preferred->present == UNKNOWN)
@@ -702,25 +666,31 @@ keyboard_select (void)
 	}
 	sercon = check_if_serial_console();
 	umlcon = check_if_uml_console();
-	none = translated_template_get ("kbd-chooser/no-keyboard");
 	if (sercon == SERIAL_PRESENT || umlcon == SERIAL_PRESENT) {
+		debconf_metaget(client, "kbd-chooser/no-keyboard", "Description");
+		arch_descr = strdup(client->value);
 		choices++;
-		s = insert_description (s, none, &first_entry);
-		mydebconf_default_set ("console-tools/archs", none);
+		s = insert_description (s, "no-keyboard", &first_entry_s);
+		t = insert_description (t, arch_descr, &first_entry_t);
+		mydebconf_default_set ("console-tools/archs", "no-keyboard");
 	} else {
-		if (((preferred == NULL) || (preferred->present == UNKNOWN))
-		    && (sercon != SERIAL_PRESENT) && (umlcon != SERIAL_PRESENT)) {
-			di_info ("Can't tell if kbd present; add no keyboard option\n");
-			choices++;
-			s = insert_description (s, none, &first_entry);
-		}
+		// Add option to skip keyboard configuration (use kernel keymap)
+		debconf_metaget(client, "kbd-chooser/skip-config", "Description");
+		arch_descr = strdup(client->value);
+		choices++;
+		s = insert_description (s, "skip-config", &first_entry_s);
+		t = insert_description (t, arch_descr, &first_entry_t);
 		mydebconf_default_set ("console-tools/archs",  
-				      preferred ? preferred->description : none);
+				      preferred ? preferred->name : "skip-config");
 	}
-	debconf_subst (client, "console-tools/archs", "choices", buf);
-	free(none);
+	debconf_subst (client, "console-tools/archs", "KBD-ARCHS", buf_s);
+	debconf_subst (client, "console-tools/archs", "KBD-ARCHS-L10N", buf_t);
+	free(arch_descr);
+	// Set medium priority if current selection is no-keyboard or skip-config
 	return ((sercon == SERIAL_PRESENT) || (umlcon == SERIAL_PRESENT) ||
-		(preferred && preferred->present == TRUE)) ? "low" : "medium";
+		((preferred && preferred->present == TRUE) &&
+		 (strcmp (curr_arch, "skip-config") != 0) &&
+		 (strcmp (curr_arch, "no-keyboard") != 0))) ? "low" : "medium";
 }
 
 /**
@@ -751,13 +721,13 @@ keymap_select (char *arch, char *keymap)
 	// This is set if we can actually read preferred type from keyboard,
 	// and shouldn't have to ask the question.
 	if (kb->deflt) {
-		def = keymap_get (maplist_get (arch), kb->deflt);
+		def = keymap_get (maplist_get (arch), kb->deflt, NULL);
 		mydebconf_default_set (template, kb->deflt);
 	}
 	res = mydebconf_ask ( kb->deflt ? "low" : "high", template, &ptr);
 	if (res != CMD_SUCCESS)
 		return res;
-	strcpy (keymap, (strlen (ptr) == 0) ? "none" : ptr);
+	strcpy (keymap, (strlen (ptr) == 0) ? "skip-config" : ptr);
 
 	return CMD_SUCCESS;
 }
@@ -774,19 +744,19 @@ keymap_set (struct debconfclient *client, char *keymap)
 	// NOTE: not a typo, using 'true' makes things fail. amck!!!
 	debconf_fset (client, "debian-installer/keymap", "seen", "yes");
 	loadkeys_wrapper (keymap);
-
 }
 
 
 int
 main (int argc, char **argv)
 {
-	char *kbd_priority, *arch = NULL, keymap[LINESIZE], buf[LINESIZE], *s;
+	char *kbd_priority, buf[LINESIZE], *arch;
+	char keymap[LINESIZE]="no keymap";
 	enum { GOBACK, CHOOSE_ARCH, CHOOSE_KEYMAP, QUIT } state = CHOOSE_ARCH;
 	struct debconfclient *client;
 
 	setlocale (LC_ALL, "");
-	client = mydebconf_get ();
+	client = mydebconf_client ();
 	di_system_init("kbd-chooser"); // enable syslog
 
 	if (argc == 2) { // keymap may be specified on command-line
@@ -798,9 +768,10 @@ main (int argc, char **argv)
 	debconf_version (client,  2);
 
 	read_keymap_files (KEYMAPLISTDIR);
-	check_if_serial_console ();
-	kbd_priority = keyboard_select ();
-	s = buf;
+	arch = buf;
+	if (debconf_get (client, "console-tools/archs") == CMD_SUCCESS)
+		arch = strdup(client->value);
+	kbd_priority = keyboard_select (arch);
 
 	while (state != QUIT)   {
 		switch (state)	{
@@ -808,17 +779,18 @@ main (int argc, char **argv)
 			di_info ("kbdchooser: GOBACK received; leaving");
 			exit (10);
 		case CHOOSE_ARCH: // First select a keyboard arch.
-			if (mydebconf_ask (kbd_priority, "console-tools/archs", &s)) {
+			if (mydebconf_ask (kbd_priority, "console-tools/archs", &arch)) {
 				state = GOBACK;
 			} else {
-				if (s == NULL || (strlen (s) == 0)) {
+				if (arch == NULL || (strlen (arch) == 0)) {
 					di_info ("kbd-chooser: not setting keymap (console-tools/archs not set)");
 					state = QUIT;
 					break;
 				}
-				arch = keyboard_parse(s);
-				if (strcmp (arch, "none") == 0)	 {
-					di_info ("kbd-chooser: not setting keymap (kbd == none selected)");
+				di_info ("kbd-chooser: arch %s selected", arch);
+				if ((strcmp (arch, "no-keyboard") == 0) ||
+				    (strcmp (arch, "skip-config") == 0))	 {
+					di_info ("kbd-chooser: not setting keymap");
 					state = QUIT;
 					break;
 				}
